﻿using System.Collections.Generic;
using System.Linq;
using uqac_ia_sudoku_csp.Interfaces;

namespace uqac_ia_sudoku_csp.Solver.NextNumberChoosers
{
    public class LcvNextNumberChooserValue
    {
        public int Value { get; set; }

        public int LegalValuesCount { get; set; }
    }

    public class LcvNextNumberChooser : INextNumberChooser
    {
        public IEnumerable<int> ChooseNext(IList<IConstraint> constraints, Board board, int x, int y)
        {
            return Enumerable.Range(0, board.Size)
                .Select(v => new LcvNextNumberChooserValue {
                    LegalValuesCount = ComputeLegalValues(constraints, board, v, x, y),
                    Value = v
                })
                .OrderBy(lcvValues => lcvValues.LegalValuesCount)
                .Select(lcvValues => lcvValues.Value);
        }

        protected int ComputeLegalValues(IList<IConstraint> constraints, Board board, int v, int x, int y)
        {
            return board.GetEmptyNeighboors(x, y)
                .Select(pair =>
                {
                    var c = 0;
                    for (var i = 0; i < board.Size; ++i)
                    {
                        board.SetNumber(x, y, i);
                        if (board.IsConsistent(x, y, constraints))
                        {
                            ++c;
                        }
                    }
                    return c;
                })
                .Aggregate(0, (i, i1) => i + i1);
        }
    }
}